#!/usr/bin/perl  
use strict;
use lib '.';
use CGI qw/:standard/;
use CGI::Carp qw(fatalsToBrowser);
use Socket;

#
#  Set the target IP and port here
#
my ($host,$port) = ('192.168.11.80',8010);

#  How many historic readings to initialize
#    Send at least the last 60 seconds, but no more than 30 minutes
#    Default is 3 minuses (180 sec)
my $n = param('N');
(!$n || $n !~ /^\d+$/ || $n<60 || $n>1800) && ($n = 180);

#
#  Start CGI output
#
STDOUT->autoflush(1);
print "Content-Type: text/event-stream\n\n";  

#
#  Connect to server
#
sub fdconnect
{
   my ($host,$port) = @_;
   my $fd;
   socket($fd , PF_INET , SOCK_STREAM , getprotobyname("tcp")) || return;
   connect($fd,sockaddr_in($port,inet_aton($host))) || return;
   binmode($fd);
   $fd->autoflush(1);
   return $fd;
}

#
#  Read n characters from specified unit
#
sub fdget
{
   my ($fd,$n,$time) = @_;
   my $buf = '';
   for (my $i=0;$i<$n;)
   {
      my $set = '';
      vec($set,fileno($fd),1) = 1;
      select($set,undef,undef,$time) || last;
      my $l = sysread($fd , $buf , $n-$i , $i);
      ($l && $l>0) || return;
      $i += $l
   }
   return $buf;
}

#
#  Read line of text
#
sub readln
{
   my ($fd) = @_;
   my $line;
   while (length(my $ch = fdget($fd,1,5.0)))
   {
      #  Return on LF
      (ord($ch)==10) && last;
      $line .= $ch;
   }
   return $line;
}

#
#  Connect to server and push history count
#
my $fd = fdconnect($host,$port);
defined($fd) || die "Cannot connect to $host:$port\n";
syswrite($fd,"$n\n");

#
#  Read data line by line and push to client
#
while (my $line = readln($fd))
{ 
   print "data: $line\r\n\r\n"; 
} 
